@@ -22,13 +22,9 @@
 extern "C" {
 #endif
 
-#define RC_PREFIX "/usr"
 #define RC_SYSCONFDIR		"/etc"
-#define RC_LIBDIR               "/usr/lib/rc"
 #define RC_LIBEXECDIR           "/usr/lib/openrc"
-#if defined(PREFIX)
-#define RC_SVCDIR               RC_LIBEXECDIR "/init.d"
-#elif defined(__linux__) || (defined(__FreeBSD_kernel__) && \
+#if defined(__linux__) || (defined(__FreeBSD_kernel__) && \
 		defined(__GLIBC__)) || defined(__GNU__)
 #define RC_SVCDIR               "/run/openrc"
 #else
@@ -37,7 +33,7 @@ extern "C" {
 #define RC_RUNLEVELDIR          RC_SYSCONFDIR "/runlevels"
 #define RC_INITDIR              RC_SYSCONFDIR "/init.d"
 #define RC_CONFDIR              RC_SYSCONFDIR "/conf.d"
-#define RC_PLUGINDIR            RC_LIBDIR "/plugins"
+#define RC_PLUGINDIR            "/usr/lib/rc/plugins"
 
 #define RC_INIT_FIFO RC_SVCDIR"/init.ctl"
 #define RC_PROFILE_ENV     RC_SYSCONFDIR "/profile.env"
@@ -59,7 +55,7 @@ extern "C" {
 #endif
 
 /* LOCAL_PREFIX is for user written stuff, which the base OS and package
- * manger don't touch. */
+ * manager don't touch. */
 #define RC_LOCAL_PREFIX		"/usr/local"
 #ifdef RC_LOCAL_PREFIX
 #  define RC_LOCAL_INITDIR      RC_LOCAL_PREFIX "/etc/init.d"
@@ -123,6 +119,40 @@ typedef TAILQ_HEAD(rc_stringlist, rc_string) RC_STRINGLIST;
 #define RC_LEVEL_SINGLE     "single"
 #define RC_LEVEL_SHUTDOWN   "shutdown"
 
+/*! Set user-wide paths for the rc_*dir functions. */
+void rc_set_user(void);
+
+/*! @return true if rc_set_user was called, false otherwise. */
+bool rc_is_user(void);
+
+/*! Get a list of all script directories containing a
+ * init.d and conf.d
+ * @return NULL terminated array of directory paths. */
+const char * const *rc_scriptdirs(void);
+
+/*! Get a list of fds for existing script directories
+ * containing a init.d and conf.d
+ * @param out pointer to store the list.
+ * @return number of fds in the list. */
+size_t rc_scriptdirfds(const int **fds);
+
+/*! The system configuration directory is where rc.conf
+ * and other configuration files are stored.
+ * @return Path to the system configuration directory */
+const char *rc_sysconfdir(void);
+
+/*! The user configuration directory is where rc.conf
+ * will be located for user-mode openrc. It is meant to
+ * override the system wide configs where applicable.
+ * @return Path to the user configuration directory, or NULL if in system mode */
+const char *rc_usrconfdir(void);
+
+/*! @return Path to runlevel directory */
+const char *rc_runleveldir(void);
+
+/*! @return Path to service directory */
+const char *rc_svcdir(void);
+
 /*! Return the current runlevel.
  * @return the current runlevel */
 char *rc_runlevel_get(void);
@@ -166,6 +196,34 @@ bool rc_runlevel_starting(void);
  * @return true if yes, otherwise false */
 bool rc_runlevel_stopping(void);
 
+enum rc_dir {
+	RC_DIR_SVCDIR,
+	RC_DIR_STARTING,
+	RC_DIR_STARTED,
+	RC_DIR_STOPPING,
+	RC_DIR_INACTIVE,
+	RC_DIR_WASINACTIVE,
+	RC_DIR_FAILED,
+	RC_DIR_HOTPLUGGED,
+	RC_DIR_DAEMONS,
+	RC_DIR_OPTIONS,
+	RC_DIR_EXCLUSIVE,
+	RC_DIR_SCHEDULED,
+	RC_DIR_INITD,
+	RC_DIR_TMP,
+	RC_DIR_SYS_MAX,
+	RC_DIR_SYSCONF = RC_DIR_SYS_MAX,
+	RC_DIR_USRCONF,
+	RC_DIR_RUNLEVEL,
+	RC_DIR_MAX,
+
+	RC_DIR_INVALID = -1
+};
+
+/*! @param runtime rc directory
+ * @return an open file descriptor, which should not be closed */
+int rc_dirfd(enum rc_dir);
+
 /*! @name RC
  * A service can be given as a full path or just its name.
  * If it's just a name then we try to resolve the service to a full path.
@@ -319,17 +377,18 @@ bool rc_service_daemons_crashed(const char *);
  * OpenRC can support some special sub system types, normally virtualization.
  * Some services cannot work in these systems, or we do something else. */
 #define RC_SYS_DOCKER  "DOCKER"
+#define RC_SYS_PODMAN  "PODMAN"
 #define RC_SYS_JAIL    "JAIL"
 #define RC_SYS_NONE    ""
 #define RC_SYS_OPENVZ  "OPENVZ"
 #define RC_SYS_LXC     "LXC"
-#define RC_SYS_PREFIX  "PREFIX"
 #define RC_SYS_RKT  "RKT"
 #define RC_SYS_SYSTEMD_NSPAWN "SYSTEMD-NSPAWN"
 #define RC_SYS_UML     "UML"
 #define RC_SYS_VSERVER "VSERVER"
 #define RC_SYS_XEN0    "XEN0"
 #define RC_SYS_XENU    "XENU"
+#define RC_SYS_WSL     "WSL"
 
 /*! Returns the type of subsystem
  * @return string from RC_SYS_* types or NULL if none detected */
@@ -601,13 +660,9 @@ typedef LIST_HEAD(rc_pidlist, rc_pid) RC_PIDLIST;
  * @return NULL terminated list of pids */
 RC_PIDLIST *rc_find_pids(const char *, const char *const *, uid_t, pid_t);
 
-/* Basically the same as rc_getline() below, it just returns multiple lines */
+/* Basically the same as getline(), it just returns multiple lines */
 bool rc_getfile(const char *, char **, size_t *);
 
-/* getline is a handy glibc function that not all libcs have, so
- * we have our own */
-ssize_t rc_getline(char **, size_t *, FILE *);
-
 /* __END_DECLS */
 #ifdef __cplusplus
 }
